#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
xml工具
参考 nodeType
    ELEMENT_NODE                = 1
    ATTRIBUTE_NODE              = 2
    TEXT_NODE                   = 3
    CDATA_SECTION_NODE          = 4
    ENTITY_REFERENCE_NODE       = 5
    ENTITY_NODE                 = 6
    PROCESSING_INSTRUCTION_NODE = 7
    COMMENT_NODE                = 8
    DOCUMENT_NODE               = 9
    DOCUMENT_TYPE_NODE          = 10
    DOCUMENT_FRAGMENT_NODE      = 11
    NOTATION_NODE               = 12
"""

import xml.dom.minidom

from pyaliyun.core import six
from pyaliyun.core.exception import PyAliParameterException

XML_ATTRIBUTE_KEY_NAME = "@attributes@"
XML_DOCUMENT_KEY_NAME = "@document@"

class XmlDecoder(object):
    """
    xml解析类
    """

    @staticmethod
    def xml_to_dict(xml_string):
        dict_root = {}
        xml_string = xml_string.strip()
        if xml_string == "":
            return dict_root
            # raise PyAliParameterException("ParamError", "Xml data is \"\"!", param_name="xml_data")
        root_document = xml.dom.minidom.parseString(xml_string)
        parse_dict = XmlDecoder.parse_childnodes(root_document)
        return parse_dict

    @staticmethod
    def parse_childnodes(node):
        node_dict = {}
        attrs_dict = XmlDecoder.parse_node_attributes(node)
        if attrs_dict:
            node_dict.update(attrs_dict)
        for sub_node in node.childNodes:
            sub_name = sub_node.nodeName
            # 如果有子节点,继续解析子节点
            if sub_node.hasChildNodes():
                child_dict = XmlDecoder.parse_childnodes(sub_node)
                if sub_name in node_dict:
                    old_sub_value = node_dict.get(sub_name)
                    if type(old_sub_value) == six.dict_type:
                        # 转成列表, 添加两个新的进去
                        sub_list = []
                        sub_list.append(old_sub_value)
                        sub_list.append(child_dict)
                        node_dict.update({sub_name: sub_list})
                    elif type(old_sub_value) == six.list_type:
                        old_sub_value.append(child_dict)
                        node_dict.update({sub_name: old_sub_value})
                else:
                    node_dict.update({sub_name: child_dict})
            else:
                node_text = XmlDecoder.parse_node_text(node)
                if node_text:
                    node_dict.update(node_text)

        return node_dict

    @staticmethod
    def parse_node_text(node):
        """
        文本节点
        :param node:
        :return:
        """
        # 只有一个子节点
        if len(node.childNodes) == 1:
            text_node = node.childNodes[0]
            if text_node.nodeName == '#text':
                node_value = text_node.nodeValue.strip()
                if node_value != '\\n':
                    return {'@document@': node_value}
                return None

    @staticmethod
    def parse_node_attributes(node):
        """
        解析节点的属性
        :param node:
        :return:
        """
        node_attrs = node.attributes
        if node_attrs:
            attr_dict = {}
            for attr in node_attrs.items():
                attr_dict[attr[0]] = attr[1]
            return {'@attributes@': attr_dict}
        return None

    @staticmethod
    def get_attributes(node_dict):
        """
        获取属性的字典
        :param node_dict:
        :return:
        """
        return node_dict.get(XML_ATTRIBUTE_KEY_NAME)

    @staticmethod
    def get_document(node_dict):
        """
        获取节点里的纯文本
        :param node_dict:
        :return:
        """
        return node_dict.get(XML_DOCUMENT_KEY_NAME)


#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
aliyun的辅助工具方法
"""
import hmac
import time
import xml.dom.minidom
from hashlib import sha1 as sha

from pyaliyun.core.cryptool import b64encode_as_string
from pyaliyun.core.encoding import force_text, force_bytes
from pyaliyun.core.six import iteritems, dict_type


def get_date():
    """
    获取标准GMT时间格式,放在header里用
    :return:
    """
    return time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime())


def sign(
        ak_secret,
        http_method,
        content_md5,
        content_type,
        date,
        resource_string,
        headers_string):
    """
    通用签名函数
    """
    # 连接待签字符串
    string_to_sign = '\n'.join([http_method,
                                content_md5,
                                content_type,
                                date,
                                headers_string + resource_string])
    h = hmac.new(force_bytes(ak_secret), force_bytes(string_to_sign), sha)
    signature = b64encode_as_string(h.digest()).strip()
    # signature = base64.b64decode(force_bytes(h.digest())).strip()
    return signature


def build_header_authorization(business, ak_id, signature):
    """
    组合最终Authorization的值,用以放在header里
    Authorization: MNS AccessKeyId:Signature
    :param business:
    :param ak_id:
    :param signature:
    :return:
    """
    return "{0} {1}:{2}".format(business, ak_id, signature)


def buiild_canonicalized_headers(headers, prefix):
    """
    aliyun的header里的canonicalized属性排序和计算
    :param headers:
    :param prefix:
    :return:
    """
    canonicalized_headers = ""
    if len(headers) > 0:
        x_header_list = headers.keys()
        sorted(x_header_list)
        for k in x_header_list:
            if k.startswith(prefix):
                canonicalized_headers += k + ":" + headers[k] + "\n"
    return canonicalized_headers


def dictToXml(params, xmlns):
    """array转xml"""
    xml = [u"<?xml version=\"1.0\" encoding=\"UTF-8\"?>"]
    for k, v in iteritems(params):
        if isinstance(v, int) or isinstance(v, float):
            xml.append(u"<{0}>{1}</{0}>".format(k, v))
        elif v.isdigit():
            xml.append(u"<{0}>{1}</{0}>".format(k, v))
        else:
            text_v = force_text(v)
            xml.append(u"<{0}><![CDATA[{1}]]></{0}>".format(k, text_v))
    xml_body = u"".join(xml)
    xml_body_utf8 = xml_body.encode('utf-8')
    return xml_body_utf8


def dict_to_xml(tag_name, data_dic, xmlns):
    """
    编译成xml
    :param tag_name:
    :param data_dic:
    :param xmlns:
    :return:
    """
    doc = xml.dom.minidom.Document()
    rootNode = doc.createElement(tag_name)
    rootNode.attributes["xmlns"] = xmlns
    doc.appendChild(rootNode)
    if data_dic:
        for k, v in data_dic.items():
            keyNode = doc.createElement(k)
            if type(v) is dict_type:
                for subkey, subv in v.items():
                    subNode = doc.createElement(subkey)
                    subNode.appendChild(doc.createTextNode(subv))
                    keyNode.appendChild(subNode)
            else:
                keyNode.appendChild(doc.createTextNode(force_text(v)))
            rootNode.appendChild(keyNode)
    else:
        nullNode = doc.createTextNode("")
        rootNode.appendChild(nullNode)
    return doc.toxml("utf-8")



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
参考: https://help.aliyun.com/product/27412.html?spm=5176.doc35134.3.1.vkdmiJ
"""
import base64

from pyaliyun.core.api import BaseApi
from pyaliyun.core.encoding import force_bytes
from pyaliyun.core.request import AliRequest
from pyaliyun.core.utils import dict_to_xml


class MnsBaseApi(BaseApi):
    """
    消息服务 api
    """
    def __init__(self, host, account, schema="https", service="MNS"):
        """
        :param account:
        :param endpoint: http(s)://1698467070378510.mns.cn-hangzhou.aliyuncs.com/
        :param service:
        """
        self._version = "2015-06-06"
        self._content_type = "XML"
        self._xmlns = "http://mns.aliyuncs.com/doc/v1/"
        self._host = host
        self._schema = schema
        self._endpoint = "%s://%s" % (self._schema, self._host)
        self._header_prefix = "x-mns-"
        super(MnsBaseApi, self).__init__(account, service)

    def process_post_body(self, data, data_tag_name):
        if self._content_type == "XML":
            return dict_to_xml(data_tag_name, data, self._xmlns)
        else:
            return str(data)

    def get_request(self, resource):
        request = AliRequest(
            host=self._host,
            ak_id=self._account.access_key,
            ak_secret=self._account.access_secret,
            resource=resource,
            aliyun_service=self._service,
            header_prefix=self._header_prefix,
            method="POST",
            scheme="https"
        )
        request.add_headers({"x-mns-version": self._version})
        return request

class MnsQueueApi(MnsBaseApi):
    """
    消息服务 队列api
    """


    def send_message(self, queue_name, message_body, delay=0, priority=8):
        """
        https://help.aliyun.com/document_detail/35134.html?spm=5176.doc32295.6.686.ZZ6M35
        发送消息
        :return:
        """
        resource = '/queues/%s/messages' % queue_name
        origin_data = {
            "MessageBody": base64.b64encode(force_bytes(message_body)),
            "DelaySeconds": str(delay),
            "Priority": str(priority)
        }
        post_data = self.process_post_body(origin_data, "Message")
        ali_resp = self.post(post_data, resource)
        return ali_resp

    def __str__(self):
        return "MNS API SERVICE"

    def receive_message(self, queue_name, waitseconds=None):
        """
        消费消息
        https://help.aliyun.com/document_detail/35136.html?spm=5176.doc35131.6.688.rmU1qQ
        :param queue_name:
        :return:
        """
        resource = '/queues/%s/messages' % queue_name
        if waitseconds:
            resource = '/queues/%s/messages?waitseconds=%d' % (queue_name, int(waitseconds))
        ali_resp = self.get(resource)
        return ali_resp

    def delete_message(self, queue_name, receipt_handle):
        """
        删除消息
        :param test_queue:
        :param receipt_handle:
        :return:
        """
        resource = '/queues/%s/messages?ReceiptHandle=%s' % (queue_name, receipt_handle)
        ali_resp = self.delete(resource)
        return ali_resp


class MnsTopicApi(MnsBaseApi):
    """
    主题操作api
    """

    def send_message(self, topic_name, message_body, message_tag=None, message_attributes=None):
        """
        https://help.aliyun.com/document_detail/27497.html?spm=5176.doc27495.6.699.jm3t0R
        发送消息
        :return:
        """
        resource = '/topics/%s/messages' % topic_name
        origin_data = {
            "MessageBody": base64.b64encode(force_bytes(message_body)),
        }
        if message_tag:
            origin_data.update({"MessageTag": message_tag})
        if message_attributes:
            origin_data.update({"MessageAttributes": message_attributes})
        post_data = self.process_post_body(origin_data, "Message")
        ali_resp = self.post(post_data, resource)
        return ali_resp



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import datetime
from decimal import Decimal

from pyaliyun.core import six

if six.PY3:
    pass

python_2_unicode_compatible = six.python_2_unicode_compatible

_PROTECTED_TYPES = six.integer_types + (
    type(None), float, Decimal, datetime.datetime, datetime.date, datetime.time
)


class WcUnicodeDecodeError(UnicodeDecodeError):
    def __init__(self, obj, *args):
        self.obj = obj
        UnicodeDecodeError.__init__(self, *args)

    def __str__(self):
        original = UnicodeDecodeError.__str__(self)
        return '%s. You passed in %r (%s)' % (original, self.obj, type(self.obj))


def smart_bytes(s, encoding='utf-8', strings_only=False, errors='strict'):
    """
    Returns a bytestring version of 's', encoded as specified in 'encoding'.

    If strings_only is True, don't convert (some) non-string-like objects.
    """
    return force_bytes(s, encoding, strings_only, errors)

def smart_text(s, encoding='utf-8', strings_only=False, errors='strict'):
    """
    Returns a text object representing 's' -- unicode on Python 2 and str on
    Python 3. Treats bytestrings using the 'encoding' codec.

    If strings_only is True, don't convert (some) non-string-like objects.
    """
    return force_text(s, encoding, strings_only, errors)

def force_text(s, encoding='utf-8', strings_only=False, errors='strict'):
    """
    Similar to smart_text, except that lazy instances are resolved to
    strings, rather than kept as lazy objects.

    If strings_only is True, don't convert (some) non-string-like objects.
    """
    # Handle the common case first for performance reasons.
    if issubclass(type(s), six.text_type):
        return s
    if strings_only and is_protected_type(s):
        return s
    try:
        if not issubclass(type(s), six.string_types):
            if six.PY3:
                if isinstance(s, bytes):
                    s = six.text_type(s, encoding, errors)
                else:
                    s = six.text_type(s)
            elif hasattr(s, '__unicode__'):
                s = six.text_type(s)
            else:
                s = six.text_type(bytes(s), encoding, errors)
        else:
            # Note: We use .decode() here, instead of six.text_type(s, encoding,
            # errors), so that if s is a SafeBytes, it ends up being a
            # SafeText at the end.
            s = s.decode(encoding, errors)
    except UnicodeDecodeError as e:
        if not isinstance(s, Exception):
            raise WcUnicodeDecodeError(s, *e.args)
        else:
            # If we get to here, the caller has passed in an Exception
            # subclass populated with non-ASCII bytestring data without a
            # working unicode method. Try to handle this without raising a
            # further exception by individually forcing the exception args
            # to unicode.
            s = ' '.join(force_text(arg, encoding, strings_only, errors)
                         for arg in s)
    return s


def force_bytes(s, encoding='utf-8', strings_only=False, errors='strict'):
    """
    Similar to smart_bytes, except that lazy instances are resolved to
    strings, rather than kept as lazy objects.

    If strings_only is True, don't convert (some) non-string-like objects.
    """
    # Handle the common case first for performance reasons.
    if isinstance(s, bytes):
        if encoding == 'utf-8':
            return s
        else:
            return s.decode('utf-8', errors).encode(encoding, errors)
    if strings_only and is_protected_type(s):
        return s
    if not isinstance(s, six.string_types):
        try:
            if six.PY3:
                return six.text_type(s).encode(encoding)
            else:
                return bytes(s)
        except UnicodeEncodeError:
            if isinstance(s, Exception):
                # An Exception subclass containing non-ASCII data that doesn't
                # know how to print itself properly. We shouldn't raise a
                # further exception.
                return b' '.join(force_bytes(arg, encoding, strings_only, errors)
                                 for arg in s)
            return six.text_type(s).encode(encoding, errors)
    else:
        return s.encode(encoding, errors)


if six.PY3:
    smart_str = smart_text
    force_str = force_text
else:
    smart_str = smart_bytes
    force_str = force_bytes
    # backwards compatibility for Python 2
    smart_unicode = smart_text
    force_unicode = force_text


def is_protected_type(obj):
    """Determine if the object instance is of a protected type.

    Objects of protected types are preserved as-is when passed to
    force_text(strings_only=True).
    """
    return isinstance(obj, _PROTECTED_TYPES)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
阿里云的响应包装
    提供以下方便的属性
    service 服务, 如 MNS
    interface 业务, 如CreateQueue
    host_id 主机名
    request_id  请求id
    error_code 直接拿出错误代码比如 UnsupportedOperation
    error_msg  错误消息 如 The specified action is not supported.
    is_success 是否成功 True or False
    content_data 转换后的数据 xml转成dict, json的数据就是json
    prefix header里的前缀 如 x-oss-

    其他属性
    status_code
    encoding
    content_type
"""
from pyaliyun.core.constant import ALI_CONTENT_TYPE_XML, ALI_CONTENT_TYPE_JSON
from pyaliyun.core.exception import PyAliErrorResponseException
from pyaliyun.core.xmltools import XmlDecoder


class AliResponse(object):
    def __init__(self, service, resp):
        self.service = service
        self._resp = resp
        self.headers = resp.headers
        self.header_prefix = "x-%s-" % str(service).lower()

        self.error_code = None
        self.error_msg = None
        self.host_id = None
        self.request_id = None

        self._process()

    def _process(self):
        """
        处理数据
        :return:
        """
        # 数据类型
        self._process_result()
        self._process_content()
        self._process_general_info()

    def _process_general_info(self):
        """
        处理其他通用信息
        :return:
        """
        if not self.is_success:
            # 不成功,提取错误消息等等数据
            if self.content_type in ALI_CONTENT_TYPE_XML:
                self.error_code = XmlDecoder.get_document(self.content_data['Error']['Code'])
                self.error_msg = XmlDecoder.get_document(self.content_data['Error']['Message'])
                self.host_id = XmlDecoder.get_document(self.content_data['Error']['HostId'])
                self.request_id = XmlDecoder.get_document(self.content_data['Error']['RequestId'])
            if self.content_type in ALI_CONTENT_TYPE_JSON:
                self.error_code = self.content_data['Code']
                self.error_msg = self.content_data['Message']
                self.host_id = self.content_data['HostId']
                self.request_id = self.content_data['RequestId']
        else:
            # 成功之后
            request_id_header_key = "%srequest-id" % self.header_prefix
            # 先从header里检查
            if request_id_header_key in self.headers:
                self.request_id = self.headers[request_id_header_key]
            if self.content_type in ALI_CONTENT_TYPE_XML:
                # self.host_id = XmlDecoder.get_document(self.content_data['Error']['HostId'])
                # self.request_id = XmlDecoder.get_document(self.content_data['Error']['RequestId'])
                pass
            if self.content_type in ALI_CONTENT_TYPE_JSON:
                # json的有的在content里
                if 'RequestId' in self.content_data:
                    self.request_id = self.content_data['RequestId']

    def _process_result(self):
        """
        判断结果
        :return:
        """
        self.status_code = self._resp.status_code
        if self.status_code >= 300:
            self.is_success = False
        elif 300 > self.status_code >= 200:
            self.is_success = True
        else:
            raise PyAliErrorResponseException("ErrorStatus", "Unknown Status Code", code=self.status_code)

    def _process_content(self):
        """
        内容数据处理
        :return:
        """
        self.encoding = self._resp.encoding
        self.content_type = self.headers['content-type']
        if self.content_type in ALI_CONTENT_TYPE_XML:
            self.content_data = XmlDecoder.xml_to_dict(self._resp.text)
        elif self.content_type in ALI_CONTENT_TYPE_JSON:
            self.content_data = self._resp.json



#!/usr/bin/python
# -*- coding: utf-8 -*-
import logging

from pyaliyun.core.response import AliResponse

logger = logging.getLogger(__name__)


class BaseApi(object):
    def __init__(self, account, service=None, response_class=None):
        """
        service 为签名用到的如  OSS MNS 等等服务的名字
        :param account:
        :param service:
        """
        self._account = account
        self._service = service
        if not response_class:
            self.response_class = AliResponse
        else:
            self.response_class = response_class

    def get_request(self, resource):
        raise NotImplementedError

    def post(self, post_data, resource):
        req = self.get_request(resource)
        resp = req.post(post_data)
        ali_resp = self.response_class(self._service, resp)
        return ali_resp

    def get(self, resource):
        req = self.get_request(resource)
        resp = req.get()
        ali_resp = self.response_class(self._service, resp)
        return ali_resp

    def delete(self, resource):
        req = self.get_request(resource)
        resp = req.delete()
        ali_resp = self.response_class(self._service, resp)
        return ali_resp



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
包装服务信息
"""


class AliService(object):
    def __init__(self, name):
        self.name = name

    def get_header_prefix(self):
        raise NotImplementedError

    def get_business(self):
        raise NotImplementedError



#!/usr/bin/python
# -*- coding: utf-8 -*-

# 返回类型
ALI_CONTENT_TYPE_XML = ["application/xml;charset=utf-8", "text/xml;charset=utf-8",
                        "application/xml", "text/xml"]
ALI_CONTENT_TYPE_JSON = ["application/json", "text/json"]

# 注册的服务
ALI_API_SERVICE_MNS = "MNS"
ALI_API_SERVICE_OSS = "OSS"



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
加密解密的工具
"""

import base64
import hashlib

from pyaliyun.core.encoding import force_bytes, force_text

__author__ = "vincent"
__email__ = "ohergal@gmail.com"
__copyright__ = "Copyright 2015, tiqiua.com"


def b64encode_as_string(data):
    """
    base64编码后返回字符串
    :param data:
    :return:
    """
    return force_text(base64.b64encode(data))


def content_md5(data):
    """
    http的body计算content的md5的值, 返回字符串
    :param data:
    :return:
    """
    m = hashlib.md5(force_bytes(data))
    return b64encode_as_string(m.digest())


def md5_string(data):
    """
    返回md5编码的字符串
    :param data:
    :return:
    """
    return hashlib.md5(force_bytes(data)).hexdigest()



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
请求包装
"""

import requests

from pyaliyun.core.cryptool import content_md5
from pyaliyun.core.response import AliResponse
from .utils import buiild_canonicalized_headers, sign, get_date, build_header_authorization


class AliRequest:
    """
    基础请求包装类
    """

    def __init__(self,
                 host,
                 ak_id,
                 ak_secret,
                 resource,
                 aliyun_service,
                 content_type="text/xml;charset=UTF-8",
                 header_prefix=None,
                 headers=None,
                 method="GET",
                 scheme="http"):
        if headers is None:
            headers = {}
        self.host = host
        self.ak_id = ak_id
        self.ak_secret = ak_secret
        self.resource = resource
        self.aliyun_service = aliyun_service
        self.content_type = content_type
        self.headers = headers
        self.header_prefix = header_prefix
        self.request_body = None
        self.method = method
        self.scheme = scheme
        # 判断是否build完成, 初始化为False
        self.built = False
        self.request_url = "%s://%s%s" % (self.scheme, self.host, self.resource)

    def add_headers(self, data):
        """
        添加header值
        :param data:
        :return:
        """
        self.headers.update(data)

    def _sign(self, date, md5_hash):
        """
        签名
        :return:
        """

        canonicalized_headers = buiild_canonicalized_headers(self.headers, self.header_prefix)
        signature = sign(self.ak_secret,
                         self.method, md5_hash, self.content_type, date, self.resource, canonicalized_headers)
        return signature

    def _build(self):
        """
        构建请求
        :return:
        """
        date_string = get_date()
        if self.request_body:
            content_length = str(len(self.request_body))
        else:
            content_length = "0"
        md5_hash = ""
        if self.method == "POST":
            md5_hash = content_md5(self.request_body)
        self.headers.update(
            {
                "Content-Type": self.content_type,
                "Content-Length": content_length,
                "Content-Md5": md5_hash,
                "Date": date_string,
                "Host": self.host
            })
        signature = self._sign(date_string, md5_hash)
        authorization = build_header_authorization(self.aliyun_service, self.ak_id, signature)
        self.headers.update({"Authorization": authorization})
        self.built = True

    def _precondition(self):
        self._build()
        assert self.built, "Request has not been build"

    def post(self, data):
        """
        发送post请求
        :return:
        """
        self.method = "POST"
        self.request_body = data
        self._precondition()
        return requests.post(self.request_url, headers=self.headers, data=data)

    def get(self):
        """
        发送get请求
        :return:
        """
        self.method = "GET"
        self._precondition()
        return requests.get(self.request_url, headers=self.headers)

    def delete(self):
        """
        delete请求
        :return:
        """
        self.method = "DELETE"
        self._precondition()
        return requests.delete(self.request_url, headers=self.headers)



from distutils.core import setup

from setuptools import find_packages


packages = [
    'pyaliyun',
    'pyaliyun.core',
    'pyaliyun.mns',
]

setup(
    name='pyaliyun',
    version='1.0.4.3',
    # packages=['pyaliyun'],
    packages=packages,
    url='http://www.wantchalk.com',
    # package_dir={'pyaliyun': 'pyaliyun'},
    license='MIT',
    author='vincent wantchalk',
    author_email='ergal@163.com',
    description='Python language sdk and tools for Aliyun service api',
    # What does your project relate to?
    keywords='aliyun oss mns',
    install_requires=['requests>=2.11.1'],
    classifiers=[
        # How mature is this project? Common values are
        # Development Status:: 1 - Planning
        # Development Status:: 2 - Pre - Alpha
        # Development Status:: 3 - Alpha
        # Development Status:: 4 - Beta
        # Development Status:: 5 - Production / Stable
        # Development Status:: 6 - Mature
        # Development Status:: 7 - Inactive
        'Development Status :: 3 - Alpha',
        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries',
        # OS
        'Operating System :: OS Independent',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

)



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
ram和sts的支持的产品列表
https://help.aliyun.com/knowledge_detail/39742.html?spm=5176.7839741.2.1.SFyJQf

"""

import logging

logger = logging.getLogger(__name__)


class Account(object):
    """
    账户类 里面保存ak等信息
    account_type M为主账户 S为子账户
    account_name 可以不填, 不填代表是主账户
    """

    def __init__(self,
                 access_key,
                 access_secret,
                 account_name=None,
                 account_type='M'):
        self.access_key = access_key
        self.access_secret = access_secret
        self.account_name = account_name
        self.account_type = account_type



#!/usr/bin/python
# -*- coding: utf-8 -*-


class PyAliException(Exception):
    """
    基础异常类
    """

    def __init__(self, error_type, message, req_id=None):
        self.error_type = error_type
        self.message = message
        self.req_id = req_id

    def get_error_info(self):
        if self.req_id is not None:
            return "(\"%s\" \"%s\") RequestID:%s\n" % (self.error_type, self.message, self.req_id)
        else:
            return "(\"%s\" \"%s\")\n" % (self.error_type, self.message)

    def __str__(self):
        return "AliyunException  %s" % (self.get_error_info())


class PyAliParameterException(PyAliException):
    """
    参数错误
    """

    def __init__(self, error_type, message, req_id=None, param_name=None):
        self.param_name = param_name
        super(PyAliParameterException, self).__init__(error_type, message, req_id)

    def get_error_info(self):
        if self.req_id is not None:
            return "(\"%s\" \"%s\") RequestID:%s\n Param name : %s" % \
                   (self.error_type, self.message, self.req_id, self.param_name)
        else:
            return "(\"%s\" \"%s\")\n Param name : %s" % \
                   (self.error_type, self.message, self.param_name)

    def __str__(self):
        return "PyAliParameterException  %s" % (self.get_error_info())


class PyAliErrorResponseException(PyAliException):
    """
    错误的响应
    """

    def __init__(self, error_type, message, req_id=None, code=None):
        self.code = code
        super(PyAliErrorResponseException, self).__init__(error_type, message, req_id)

    def get_error_info(self):
        if self.req_id is not None:
            return "(\"%s\" \"%s\") RequestID:%s\n Error code: %s" % \
                   (self.error_type, self.message, self.req_id, self.code)
        else:
            return "(\"%s\" \"%s\")\n Error code: %s" % \
                   (self.error_type, self.message, self.code)

    def __str__(self):
        return "PyAliErrorResponseException  %s" % (self.get_error_info())


class PyAliXmlParseException(PyAliException):
    """
    xml解析错误
    """

    def __init__(self, error_type, message, req_id=None):
        super(PyAliXmlParseException, self).__init__(error_type, message, req_id)

    def __str__(self):
        return "PyAliXmlParseException  %s" % (self.get_error_info())